/* DO NOT EDIT.  This file is generated automatically. */

/*
 *  Pix
 *
 *  Copyright (C) 2010 Free Software Foundation, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>
#include <glib/gi18n.h>

static char *text[] = {
	N_("Click to view the image"),
	N_("Go back to the index"),
	N_("Image %d of %d"),
	N_("Index"),
	N_("Next"),
	N_("Page %d of %d"),
	N_("Previous"),
	N_("View page %d"),
	N_("View the next image"),
	N_("View the next page"),
	N_("View the previous image"),
	N_("View the previous page"),
	NULL
};
